/*****************************************************************************
 *****************************************************************************
 Copyright (c) 2000-2001, AEP Ltd. 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of AEP Ltd. nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

 *****************************************************************************
****************************************************************************/

/**********************************************************************
*                                                                     *
* AEP Ltd.                                                            *
*                                                                     *
* This software is supplied under the terms of the license included   *
* above.  All use of this software must be in accordance with the     *
* terms of that license.                                              *
*                                                                     *
**********************************************************************/
/****************************************************************/
/*																*/
/* Filename:	aep_api.h										*/
/* Description:	AEP API Include file, fxn prototypes for the	*/		
/*				AEP API.										*/
/* Copyright:	Accelerated Encryption Processing Ltd., 2000	*/
/* Author:		Diarmuid O'Neill								*/
/* Product:		AEP1000											*/
/* Date:		12/9/99											*/
/*																*/
/****************************************************************/

#ifndef aep_api_H 
#define aep_api_H 1

/*Header file that contains OS specific details*/
#include "aep_lib.h"

/*Header file that contains AEP API return codes and other definitions*/
#include "aep_defs.h"

/*Header file that contains AEP type definitions*/
#include "aep_types.h"

#include "aep_version.h"

/*	AEP_Initialize:
	Initialise the api.  This fxn must be called before any other API
	calls are made.  If another API call is made before calling this
	routine the call will return AEP_R_AEPAPI_NOT_INITIALIZED.

	The Init Args should be NULL.

	Inputs	:	Pointer to initargs  {pInitArgs}
	Outputs	:	None
*/
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_Initialize(
		AEP_VOID_PTR pInitArgs
	);


/* AEP_Finalize
	Indicates that the application programmer is finished using the
	AEP API.  The function frees up any storage used by the API.
		
	Inputs	:	None
	Outputs	:	None
*/
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_Finalize(
	);


/* AEP_SetBNCallBacks
	Provides the addresses of the callback functions to be used when
	dealing with Bignums.  These are provided by the application
	programmer.
		
	Inputs	:	Pointer to GetBNSize function
				Pointer to MakeAEPBigNumFunc function
				Pointer to ConverAEPBigNumFunc function
	Outputs	:	None
*/
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_SetBNCallBacks(
		AEP_RV (*GetBigNumSizeFunc)(),
		AEP_RV (*MakeAEPBigNumFunc)(),
		AEP_RV (*ConverAEPBigNumFunc)()
	);


/*	AEP_OpenConnection:
	Opens a connection to the AEP Daemon and returns a connection
	handle.    

	Inputs	: Pointer to a Connection Handle {phConnection}
	Outputs	: Filled pointer to a Connection Handle {phConnection}
*/
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_OpenConnection(
        AEP_CONNECTION_HNDL_PTR phConnection
	);


/*	AEP_CloseConnection:
	Closes a connection to the AEP Daemon given the connection
	handle.

	Inputs	: Connection Handle.  {hConnection}
	Outputs	: None
*/
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_CloseConnection(
        AEP_CONNECTION_HNDL hConnection
	);


/*	AEP_ModExp:
	Performs a modular exponentiation on the arguments given.

	Inputs	:	Connection Handle  {hConnection}
				Mod Exp Base A  {pA}
				Mod Exp Power P  {pP}
				Mod Exp Modulus  {pN}
				Pointer to a Transaction handle (either a valid ptr or NULL)  {pidTransId}

	Outputs	:	Result of the modular exp if a valid pointer was not passed 
				in pidTransId  {pResult}
				Filled Transaction handle pointer is returned if a valid pointer was 
				passed in pidTransId {pidTransId}
*/
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_ModExp(
        AEP_CONNECTION_HNDL		hConnection,
		AEP_VOID_PTR			pA,
		AEP_VOID_PTR			pP,
		AEP_VOID_PTR			pN,
		AEP_VOID_PTR			pResult,
		AEP_TRANSACTION_ID*		pidTransID
	);


/*	AEP_ModExpCrt:
	This function performs a modular exponentiation, and may use CRT
	to make the calculation quicker.

	Inputs	:	Connection Handle  {hConnection}
				Mod Exp Base A  {pA}
				Mod Exp First Modulus Factor P	{pP}
				Mod Exp Second Modulus Factor Q	{pQ}
				Mod Exp D Mod P-1   {pDmp1}
				Mod Exp D Mod Q-1	{pDmq1}
				Mod Exp Q**-1 Mod P	{pIqmp}
				Pointer to a Transaction handle (either a valid ptr or NULL)  {pidTransId}

	Outputs	:	Result of the modular exp.  {pResult}
				Transaction handle is returned if a valid pointer was passed in pidTransId
*/
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_ModExpCrt(
        AEP_CONNECTION_HNDL		hConnection,
		AEP_VOID_PTR			pA,
		AEP_VOID_PTR			pP,
		AEP_VOID_PTR			pQ,
		AEP_VOID_PTR			pDmp1,
		AEP_VOID_PTR			pDmq1,
		AEP_VOID_PTR			pIqmp,
		AEP_VOID_PTR			pResult,
		AEP_TRANSACTION_ID*		pidTransId
      );


/*	AEP_GenPrime:
	Generates a prime number of given length

	Inputs	:	Connection Handle  {hConnection}
				Length (In bits)  {Len}
				Pointer to a Transaction handle (either a valid ptr or NULL)  {pidTransId}

	Outputs	:	The prime number.  {pResult}
				Filled Transaction handle pointer is returned if a valid pointer was 
				passed in pidTransId {pidTransId}
*/
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_GenPrime(
        AEP_CONNECTION_HNDL		hConnection,
		AEP_U32					Len,
		AEP_VOID_PTR			pResult, 
		AEP_TRANSACTION_ID*		pidTransID
	);


/*	AEP_GenRandom:
	Generates a random number of given length

	Inputs	:	Connection Handle  {hConnection}
				Length (In bytes)  {Len}
				Pointer to a Transaction handle (either a valid ptr or NULL)  {pidTransId}

	Outputs	:	The random number.  {pResult}
				Filled Transaction handle pointer is returned if a valid pointer was 
				passed in pidTransId {pidTransId}
*/
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_GenRandom(
        AEP_CONNECTION_HNDL		hConnection,
		AEP_U32					Len,
		AEP_U32					Type,
		AEP_VOID_PTR			pResult,
		AEP_TRANSACTION_ID*		pidTransID
	);


/*	AEP_Encrypt:
	Encrypts a given block of plain text.

	Inputs	:	Connection Handle  {hConnection}
				Mechanism  {mMechanism}
				Key Identifier {idKey}
				Pointer to a context Identifier {pidContext}
				Plain Text {bbPlainText}
				Pointer to a Transaction handle (either a valid ptr or NULL)  {pidTransID}


	Outputs	:	Cipher Text {pbbCipherText}
				Filled Transaction handle pointer is returned if a valid pointer was 
				passed in pidTransId {pidTransId}
*/
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_Encrypt(
        AEP_CONNECTION_HNDL		hConnection,
		AEP_U32			mMechanism,
		AEP_KEY_ID				idKey,
		AEP_BYTEBLOCK			bbPlainText,
		AEP_BYTEBLOCK*			pbbCipherText,
		AEP_TRANSACTION_ID*		pidTransID
	);
	  

/*	AEP_EncryptInit:
	Initializes an encryption operation. 
	
	Inputs	:	Connection Handle  {hConnection}
				Encryption Mechanism  {mechanism}
				Encryption Key ID {idKey}
				Pointer to Context ID  {pidContext}

	Outputs	:	Pointer to Context ID.  {pidContext}				
*/
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_EncryptInit(
	    AEP_CONNECTION_HNDL		hConnection,
		AEP_U32			mMechanism,
		AEP_KEY_ID				idKey,
		AEP_CONTEXT_ID*			pidContext
	);


/*	AEP_EncryptUpdate:
	Continues a multiple-part encryption operation, 
	processing another data part. 
	
	Inputs	:	Connection Handle  {hConnection}
				Context ID  {idContext}
				Plain text	{plainText}
				Pointer to Cipher text	{pCipherText}
				Pointer to a Transaction handle (either a valid ptr or NULL)  {phTransHndl}

	Outputs	:	Length of cipher text {len}
				Cipher text {pCipherText}
				Filled Transaction handle pointer is returned if a valid pointer was 
				passed in pidTransId {pidTransId}
*/	
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_EncryptUpdate(
	    AEP_CONNECTION_HNDL		hConnection,
		AEP_CONTEXT_ID			idContext,
		AEP_BYTEBLOCK			bbPlainText,
		AEP_BYTEBLOCK*			pbbCipherText,
		AEP_TRANSACTION_ID*	pidTransID
	);


/*	AEP_EncryptFinal:
	Finishes a multiple-part encryption operation.  This invalidates the context referred
	to by Context Id.
	
	Inputs	:	Connection Handle  {hConnection}
				Context ID  {idContext}
				Plain text	{plainText}
				Pointer to Cipher text	{pCipherText}
				Pointer to a Transaction handle (either a valid ptr or NULL)  {phTransHndl}

	Outputs	:	Length of cipher text {len}
				Cipher text {pCipherText}
				Filled Transaction handle pointer is returned if a valid pointer was 
				passed in pidTransId {pidTransId}
*/
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_EncryptFinal(
	    AEP_CONNECTION_HNDL		hConnection,
		AEP_CONTEXT_ID			idContext,
		AEP_BYTEBLOCK			bbPlainText,
		AEP_BYTEBLOCK*			pbbCipherText,
		AEP_TRANSACTION_ID*		pidTransID
	);


/*	AEP_QueryReply:
	Checks to see if the result of a previous transaction is available.  If so,
	it is returned.

	Inputs	:	Connection Handle  {hConnection}
				Transaction handle  {idTransID}
				Void pointer to a result {pResult}

	Outputs	:	The result pointed to by pResult {pResult}
*/
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_AwaitReply(
        AEP_CONNECTION_HNDL		hConnection,
		AEP_TRANSACTION_ID		idTransID,
		AEP_VOID_PTR			pResult
	);


/*	AEP_AwaitReply:
	Checks to see if the result of a previous transaction is available.  If so,
	it is returned.  If not, the call blocks until the result is available.

	Inputs	:	Connection Handle  {hConnection}
				Transaction handle  {idTransID}
				Void pointer to a result {pResult}

	Outputs	:	The result pointed to by pResult  {pResult}
*/  
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_QueryReply(
        AEP_CONNECTION_HNDL		hConnection,
		AEP_TRANSACTION_ID		idTransID,
		AEP_VOID_PTR			pResult
	);
	
	
/*	AEP_Get_Version:
	Gets version information for the product

	Inputs	:	Connection Handle  {hConnection}
				

	Outputs	:	The result is an AEP_VERSION structure pointed to by pVersions  {pVersions}
*/  
AEP_SLIB_CALLING_CONVENTION AEP_RV AEP_GetVersion(
								AEP_CONNECTION_HNDL	hConnection,
								AEP_VERSION	*pVersions );

	

#endif
