/*****************************************************************************
 *****************************************************************************
 Copyright (c) 2000-2001, AEP Ltd. 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of AEP Ltd. nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

 *****************************************************************************
****************************************************************************/

/**********************************************************************
*                                                                     *
* AEP Ltd.                                                            *
*                                                                     *
* This software is supplied under the terms of the license included   *
* above.  All use of this software must be in accordance with the     *
* terms of that license.                                              *
*                                                                     *
**********************************************************************/

/* **************************************************************/
/*																*/
/* Filename:	aep_i_defs.h									*/
/* Description:													*/
/* Copyright:	Accelerated Encryption Processing Ltd., 2000	*/
/* Author:														*/
/* Product:	AEP1000												*/
/* Date:	12/6/00												*/
/*																*/
/****************************************************************/



#ifndef aep_i_defs_H 
#define aep_i_defs_H 1

#include "aep_defs.h"

#define	FALSE						0
#define	TRUE						1
#define ALL_BITS 0xFFFFFFFF

/**********Message header offset definitions**********/
/*Size of an AEP message header*/
#define HDR_SIZE			32

#define HDR_APPID_OFFSET	0	/*Application's context id, 64 bit field*/
#define HDR_TRANHNDL_OFFSET 8	/*Transaction handle, 64 bit field*/
#define HDR_CMD_OFFSET		16	/*Command identifier, 32 bit field*/
#define HDR_STATUS_OFFSET	20	/*Status field, 32 bit field*/
#define HDR_FLAGS_OFFSET	24	/*Flags, 32 bit field*/
#define HDR_DATALEN_OFFSET	28	/*Length of data message part, 32 bit field*/
#define HDR_DATA_OFFSET		32	/*Start of data in message*/
/**********************************************/

/*Used to determine if padding is needed for the message (sizeof(AEP_U64) -1)*/
#define PADDING_MASK 0x7 

/***********Internal Function return codes************/

/*Incarnation value is not valid*/
#define AEP_R_INVALID_CTX_INCARNATION	0x000000C1

/*The parent ctx id passed is not a valid ctx id*/
#define AEP_R_INVALID_CTX_PARENT		0x000000C2


/*****************************************************/


/*****************AEP Commands*****************/

#define AEP_C_MODEXP			0x00000001
#define AEP_C_MODEXPCRT			0x00000002
#define AEP_C_GENPRIME			0x00000003
#define AEP_C_GENRANDOM			0x00000005
#define AEP_C_QUERY				0x00000006
#define AEP_C_WAIT				0X00000007
#define AEP_C_ENCRYPT			0x00000008
#define AEP_C_ENCRYPT_INIT		0x00000009
#define AEP_C_ENCRYPT_UPDATE	0x0000000a
#define AEP_C_ENCRYPT_FINAL		0x0000000b
#define AEP_C_DECRYPT			0x0000000c
#define AEP_C_DECRYPT_INIT		0x0000000d
#define AEP_C_DECRYPT_UPDATE	0x0000000e
#define AEP_C_DECRYPT_FINAL		0x0000000f
#define AEP_C_HASH				0x00000010
#define AEP_C_HASH_INIT			0x00000011
#define AEP_C_HASH_UPDATE		0x00000012
#define AEP_C_HASH_FINAL		0x00000013
#define AEP_C_SIGN				0x00000014
#define AEP_C_SIGN_INIT			0x00000015
#define AEP_C_SIGN_UPDATE		0x00000016
#define AEP_C_SIGN_FINAL		0x00000017
#define AEP_C_VERIFY			0x00000018
#define AEP_C_VERIFY_INIT		0x00000019
#define AEP_C_VERIFY_UPDATE		0x0000001a
#define AEP_C_VERIFY_FINAL		0x0000001b

#define AEP_C_RNG_TEST			0x0000001c

#define AEP_C_IMPORT_KEY		0x0000001d
#define AEP_C_GENERATE_KEY		0x0000001e

#define AEP_C_AUTH_DAEMON		0x00001000
#define AEP_C_AUTH_APP			0x00002000
#define AEP_C_UNAUTH_APP		0x00003000

#define AEP_C_GET_VERSION		 0x0000001f
#define AEP_C_GET_DAEMON_VERSION 0x00000020

#define AEP_C_ALIVE				0x00001001
#define AEP_C_REQUEST_CTXS		0x00001002

#define AEP_C_LOAD_TARGET		0x00004001
#define AEP_C_BOOT_TARGET		0x00004002
#define AEP_C_SOFT_RESET		0x00004010

/**********************************************/

/******************AEP Flags*******************/

/*Mode of command synchronmous/Asynchronous*/
#define AEP_F_ASYNCMODE		0x00000001
#define AEP_F_SYNCMODE		0x00000002

/*Command type server/no context required/ context required*/
#define AEP_F_SRVCMD		0x00000010
#define AEP_F_NOCTXCMD		0x00000020
#define AEP_F_CTXCMD		0x00000030
/*Key Management Command flag*/
#define AEP_F_KEYMANCMD		0x00000040

/*Destination engine of command*/
#define AEP_F_ASIC			0x00000100
#define AEP_F_DES			0x00000200
#define AEP_F_SA			0x00000300
#define AEP_F_LOADER		0x00000400

/*Last message in block indicator.  Target sets this to indicate to
the host that the transfer of this message will result in a block being
freed on the target.  If the host is waiting for a block to become free
when it reads this message there should be one*/
#define AEP_F_LASTMSG		0x00010000

/*Destination of completed request (response)  Note if a command comes from
an crypto application a response item is set up for it in the daemon, if it
originates from the key manager it must be sent back to the keymanager.  We
don't want the daemon to think it has an associated response item and to try
and find it.*/
#define AEP_F_RESTOKEYMAN	0x00001000

/*Flag Masks*/
#define AEP_F_LUNMASK		0x00000F00
#define AEP_F_CMDMASK		0x000000F0

/**********************************************/

/*****************AEP Host-Target Msg Status*******************/

#define AEP_S_INIT							0x00010000
#define AEP_S_KEYMANAGER					0x00020000


#define AEP_S_COMPLETE						0x10010000
#define	AEP_S_BUSY							0x20000000
#define	AEP_S_FREE							0x30000000


#define AEP_S_INVALID_ARGS					AEP_R_ARGUMENTS_BAD
#define AEP_S_NO_RESOURCES					AEP_R_NO_TARGET_RESOURCES
#define AEP_S_UNKNOWN						AEP_R_TRANSACTION_HANDLE_INVALID
#define AEP_S_CLAIMED						AEP_R_TRANSACTION_CLAIMED
#define AEP_S_TIMEDOUT						AEP_R_TIMED_OUT			
#define AEP_S_FXN_NOT_IMPLEMENTED			AEP_R_FXN_NOT_IMPLEMENTED
#define AEP_S_TARGET_ERROR					AEP_R_TARGET_ERROR		/*0x10070000*/
#define AEP_S_DAEMON_ERROR					AEP_R_DAEMON_ERROR

#define AEP_S_INVALID_CTX					AEP_R_INVALID_CTX_ID

#define AEP_S_NOKEYMANAGER					AEP_R_NO_KEY_MANAGER

/***** AEP Host-Target Msg Status - DES ERRORS **********/
#define AEP_S_INVALID_DES_CTX				0x00000001
#define AEP_S_INVALID_DES_CTX_WITH_SESS_ID	0x00000002
#define AEP_S_NO_DES_SEGS					0x00000003
#define AEP_S_INVALID_DES_CHIP_SESSION_ID	0x00000004
#define AEP_S_INVALID_DES_COMMAND			0x00000005

/***** AEP Host-Target Msg Status - ASIC ERRORS *********/
#define	AEP_S_ASIC_RESOURCEPROBLEM			0x00000010

/**********************************************/

/*This constant is used to mark the API initialised*/
#define AEP_IGF_INITIALIZED			1

/*Used when generating session handles*/
#define I_MAX_HANDLE (0xffffffffUL)

#define CONNECT_RETRIES  10 /* At present, we make ten connection attemp5 */
#define CONNECT_INTERVAL 20000000 /*Nano secs (.02 of a second)*/

/*****************AEP ARCHIMEDES DES CHIP *****************/


/* Des command block synonyms. */

/* Command field */
#define DES_CHIP_ENCRYPT		0x1000
#define DES_CHIP_DECRYPT		0x0000
#define DES_CHIP_RD_CNTXT		0x2000
#define DES_CHIP_WR_CNTXT 		0x3000


/* algorithm_mode field synonyms. */
#define DES_COMMAND_BLK_NEW_KEY                 0x4000
#define DES_COMMAND_BLK_NEW_VECTOR              0x8000

#define DES_MODE_MASK                           0x3c00
#define DES_MODE_ECB                            0x0000

#define DES_ALGORITHM_MASK                      0x0380
#define PLAIN_DES_ALGORITHM                     0x0000
#define TRIPLE_DES_ALGORITHM                    0x0080
#define PIPLELINED_DES_ALGORITHM                0x0180

#ifdef WIN32
#define INITIAL_VALUE_IN_ON_CHIP_ARRAY			0xfffffffffffffffe
#else
#define INITIAL_VALUE_IN_ON_CHIP_ARRAY			0xfffffffffffffffeull
#endif
/*************************************************/

/* Defines for status of mem structures */
#define MEM_STRUCT_FREE       0
#define MEM_STRUCT_ALLOCATED  1
#define MEM_STRUCT_ENQUEUED   2

/*Size of a msg block*/
#define MSG_BLOCK_SIZE 		66560


/* Misc */
#define MAX_NUM_ON_CHIP_CONTEXTS    0x10
#define MAX_DES_SEG_SIZE            0x7fff8 /* 512k - 8 bytes */
#define NUM_DES_ENGINES             0x03
#define READ_CONTEXT_ENG_NUMB_MASK  0xffcf

/**********************************************/

#define NUM_CONTEXT_DB_STRUCTS      0x1000


/*************Board Types**********************/
#define AEP_NoModel 	0
#define AEP_500			1	/* 30 MHz ASIC/IPX @ 166MHz 	=> 500 ModEXP/Sec */
#define AEP_1000		2	/* 30 MHz ASIC/IXP @ 166MHz	=> 1000 ModEXP/Sec with CRT */	
#define AEP_2000		3	/* 60 MHz ASIC/IXP @ 200Mhz	=> 1000 ModEXP/Sec */
#define AEP_3000		4	/* 90 MHz ASIC  => xxxxx */
#define AEP_4000		5	/* Older Board Types With NVM/8M sram */ 


#define AEP_DAEMON_SOCKET "/var/run/aep_socket"

#endif
