/*****************************************************************************
 *****************************************************************************
 Copyright (c) 2000-2001, AEP Ltd. 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of AEP Ltd. nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

 *****************************************************************************
****************************************************************************/

/**********************************************************************
*                                                                     *
* AEP Ltd.                                                            *
*                                                                     *
* This software is supplied under the terms of the license included   *
* above.  All use of this software must be in accordance with the     *
* terms of that license.                                              *
*                                                                     *
**********************************************************************/


/* **************************************************************/
/*								*/
/* Filename:	aep_I_types.h					*/
/* Description:							*/
/* Copyright:	Accelerated Encryption Processing Ltd., 2000	*/
/* Author:							*/
/* Product:	AEP1000						*/
/* Date:	12/6/00						*/
/*								*/
/****************************************************************/



#ifndef aep_i_types_H 
#define aep_i_types_H 1

/*Header file that contains OS specific details*/
#include "aep_lib.h"

#include "aep_types.h"


typedef struct AEP_I_CONNECTION_DATA * AEP_I_CONNECTION_DATA_PTR;
typedef struct AEP_I_RESPONSE_DATA * AEP_I_RESPONSE_DATA_PTR;
typedef struct AEP_MSG_HDR * AEP_MSG_HDR_PTR;

/*Ensure the msg structure is word aligned*/
/*#pragma align 4 (AEP_MSG_HDR) */

typedef struct AEP_MSG_HDR {
	AEP_U64		app_id;
	AEP_U64		trans_hndl;
	AEP_U32		command;
	AEP_U32		status;
	AEP_U32		flags;
	AEP_U32		datalen;
} AEP_MSG_HDR;


/*****************************************************************/
/* Misc msg structs. */

typedef struct AEP_MSG_ENC_INI_REQ_SH {
    AEP_U64				context_id;
	AEP_MECHANISM		mechanism;
	AEP_U64				key_context_id;
} AEP_MSG_ENC_INI_REQ_SH;

/* Note: No definition of AEP_MSG_ENC_INI_RES_SH since no SH contained in
the response msg.*/


typedef struct AEP_MSG_ENC_MULTI_REQ_SH {
    AEP_U64     context_id;
	AEP_U32		datalen;
} AEP_MSG_ENC_MULTI_REQ_SH;

typedef struct AEP_MSG_ENC_MULTI_RES_SH {
	AEP_U32		datalen;
} AEP_MSG_ENC_MULTI_RES_SH;


/* End of Misc msg structs. */
/*****************************************************************/


/*****************************************************************/
/* Import key messages. */


typedef struct AEP_TRIPLE_DES_KEY {
	AEP_U64		key1;
	AEP_U64		key2;
	AEP_U64		key3;
} AEP_TRIPLE_DES_KEY;

typedef struct AEP_PLAIN_DES_KEY {
	AEP_U64		key;
} AEP_PLAIN_DES_KEY;

typedef union AEP_DES_KEY {
	AEP_TRIPLE_DES_KEY	triple_des;
	AEP_PLAIN_DES_KEY	plain_des;
}	AEP_DES_KEY;


typedef struct AEP_MSG_IMP_KEY_REQ_SH {
	AEP_U32			key_type;
	AEP_DES_KEY		des_key;	
} AEP_MSG_IMP_KEY_REQ_SH;


typedef struct AEP_MSG_IMP_KEY_RES_SH {
	AEP_U64		key_context_id;
} AEP_MSG_IMP_KEY_RES_SH;


/* End of Import key messages. */
/*****************************************************************/



typedef struct AEP_I_EXT_FUNCTION_LIST {
  AEP_RV	(*AEP_I_GetBigNumSize)
				(void* ArbBigNum, AEP_U32* BigNumSize);
  AEP_RV	(*AEP_I_MakeAEPBigNum)
				(void* ArbBigNum, AEP_U32 BigNumSize, unsigned char* AEP_BigNum);
  AEP_RV	(*AEP_I_ConvertAEPBigNum)
				(void* ArbBigNum, AEP_U32 BigNumSize, unsigned char* AEP_BigNum);
} AEP_I_EXT_FUNCTION_LIST;

#endif
