/*****************************************************************************
 *****************************************************************************
 Copyright (c) 2000-2001, AEP Ltd. 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of AEP Ltd. nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

 *****************************************************************************
****************************************************************************/

/**********************************************************************
*                                                                     *
* AEP Ltd.                                                            *
*                                                                     *
* This software is supplied under the terms of the license included   *
* above.  All use of this software must be in accordance with the     *
* terms of that license.                                              *
*                                                                     *
**********************************************************************/

#ifndef aep_mem_H 
#define aep_mem_H 1

#include "aep_i_defs.h"
#include "aep_i_types.h"

#define MEM_POOL_SIZE (1024*1024*7) + (1024 * 512)


#define NUM_DES_MSG_BLOCKS 		1
#define NUM_ASIC_MSG_BLOCKS 	12
#define NUM_SA_MSG_BLOCKS 		12
#define NUM_ASIC_MSG_STRUCTS 	512
#define NUM_DES_MSG_STRUCTS		32
#define NUM_SA_MSG_STRUCTS		512
#define NUM_DES_SEG_STRUCTS		1024

#define DONE_ARRAY_WATER_MARK	1

/* Ptr Value to indicate a ptr is non null but is still not valid. 
   Use the invalid instruction vector value.
   */
#define INVALID_PTR_VALUE     (void *) 0x00000004


/*MSG Structure to hold incomming MSG blocks*/
typedef struct AEP_MSG_BLOCK {
	AEP_U32		status;  				/*Used/Free indicator*/
	AEP_U32		num_bytes;				/*Number of bytes in block*/
	volatile	AEP_U32		num_messages;/*Number of unreturned messages in block*/
	AEP_U8		buf[MSG_BLOCK_SIZE];	/*Block buffer*/
} AEP_MSG_BLOCK;


/*AEP Message Structure*/
/* 18-Aug-2000 -- DO NOT CHANGE THIS DEFINITION WITHOUT INFORMING
         BOTH THE DEVELOPER FOR THE HOST SOFTWARE, AND THE MICROCODE
		   DEVELOPER*/
/*Note result needs to be on a DWORD boundary*/
typedef struct AEP_MSG_STRUCT {
	AEP_MSG_HDR*		msg_hdr;			/*00Pointer to the message header*/
	AEP_U8*				key_ptr;			/*04Pointer to the key.  May possibly change to context ptr*/
	AEP_U8*				result;				/*08Pointer to the result data*/
	AEP_U8*				data;				/*0cPointer to the messages data*/
	volatile AEP_U8		status;				/*10Used/Free indicator*/	
	AEP_U8				num_segs_to_send;	/*11Number of segments to send to device*/
	AEP_U8				num_segs_sent;		/*12Number of segs sent to device*/
	AEP_U8				num_segs_recd;		/*13Number of segs received from device*/
	AEP_MSG_BLOCK*		msg_block;			/*14Pointer to the msg block that contains this msg struct*/
} AEP_MSG_STRUCT;


typedef struct DMAmsg {
	int command;
	int address;
	int length;
}	DMAmsg;

typedef struct DMAdescriptor {
	int NumBytes;
	int PCIAddr;
	int SDRAMAddr;
	int NextDesciptor;
}	DMAdescriptor;

/* Function declaration */

/*Initialise various memory structures*/


/*AEP memory/block management function declerations*/
void InitMem( void );

AEP_MSG_BLOCK* GetMSGBlock(void);

void FreeMSGBlock(AEP_MSG_BLOCK* msg_block);

/*Message struct manipulation functions*/
AEP_MSG_STRUCT* GetAsicMsgStruct(void);

AEP_MSG_STRUCT* GetDesMsgStruct(void);

AEP_MSG_STRUCT* GetSaMsgStruct(void);

void FreeMsgStruct(AEP_MSG_STRUCT* msg_struct_ptr);

/*Done message array functions*/
AEP_RV AddMsgToDoneArray(AEP_MSG_STRUCT* msg_struct_ptr);

volatile AEP_MSG_STRUCT* GetMsgFromDoneArray(void);

/* routines to lock out u engs from sram access. */
extern void ModifyAndUnlock ( AEP_U32 *pr_ptr, AEP_U32 value);
extern void ReadAndLock ( AEP_U32 *pr_ptr, AEP_U32 *pr_value );

/* End Function declaration */

#endif

