/*****************************************************************************
 *****************************************************************************
 Copyright (c) 2000-2001, AEP Ltd. 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of AEP Ltd. nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

 *****************************************************************************
****************************************************************************/

/**********************************************************************
*                                                                     *
* AEP Ltd.                                                            *
*                                                                     *
* This software is supplied under the terms of the license included   *
* above.  All use of this software must be in accordance with the     *
* terms of that license.                                              *
*                                                                     *
**********************************************************************/


/* **************************************************************/
/*																*/
/* Filename:	aep_types.h										*/
/* Description:													*/
/* Copyright:	Accelerated Encryption Processing Ltd., 2000	*/
/* Author:														*/
/* Product:	AEP1000												*/
/* Date:	12/6/00												*/
/*																*/
/****************************************************************/


#ifndef aep_types_H 
#define aep_types_H 1

/* an unsigned 8-bit value */
typedef unsigned char		AEP_U8;

/* an unsigned 8-bit character */
typedef char				AEP_CHAR;

/* a BYTE-sized Boolean flag */
typedef AEP_U8				AEP_BBOOL;

/*Unsigned value, at least 16 bits long*/
typedef unsigned short  	AEP_U16;

/* an unsigned value, at least 32 bits long */
typedef unsigned int		AEP_U32;

#if defined(AEP_Win32)
/* 64 bit unsigned value */
typedef unsigned _int64		AEP_U64;

#elif defined(AEP_GENERIC)
/* 64 bit unsigned value */
typedef unsigned long long	AEP_U64;
#endif

/* at least 32 bits; each bit is a Boolean flag */
typedef AEP_U32			AEP_FLAGS;

typedef AEP_U8	    	*AEP_U8_PTR;
typedef AEP_CHAR    	*AEP_CHAR_PTR;
typedef AEP_U32			*AEP_U32_PTR;
typedef AEP_U64			*AEP_U64_PTR;
typedef void        	*AEP_VOID_PTR;

/* Pointer to a AEP_VOID_PTR-- i.e., pointer to pointer to void */
typedef AEP_VOID_PTR 	*AEP_VOID_PTR_PTR;

/*Used to identify an AEP connection handle*/
typedef AEP_U32					AEP_CONNECTION_HNDL;

/*Pointer to an AEP connection handle*/
typedef AEP_CONNECTION_HNDL 	*AEP_CONNECTION_HNDL_PTR;

/*Used to identify an individual transaction in the daemon*/
typedef AEP_U64					AEP_TRANSACTION_HNDL;

/*Pointer to a transaction*/
typedef AEP_TRANSACTION_HNDL	*AEP_TRANSACTION_HNDL_PTR;

/*Used by an application (in conjunction with the apps process id) to 
identify an individual transaction*/
typedef AEP_U32					AEP_TRANSACTION_ID;

/*Pointer to an applications transaction identifier*/
typedef AEP_TRANSACTION_ID 		*AEP_TRANSACTION_ID_PTR;

/*Used to identify a key on the hardware*/
typedef AEP_U64					AEP_KEY_ID;

/*Used to identify a context on the hardware*/
typedef AEP_U64					AEP_CONTEXT_ID;

/*Return value type*/
typedef AEP_U32					AEP_RV;


/*Type used for blocks of data.  Length and pointer*/
typedef struct AEP_BYTEBLOCK {
	AEP_U32		Len;
	AEP_U8_PTR	ptr;
} AEP_BYTEBLOCK;

/*****************************************************************/
/* Structs for mechansim. */

typedef struct AEP_TRIPLE_DES_INIT_VECTOR {
	AEP_U64             init_vector_1;
	AEP_U64             init_vector_2;
	AEP_U64             init_vector_3;
} AEP_TRIPLE_DES_INIT_VECTOR;

typedef struct AEP_PLAIN_DES_INIT_VECTOR {
	AEP_U64             init_vector;
} AEP_PLAIN_DES_INIT_VECTOR;


typedef union AEP_MECHANISM_DATA {
	AEP_TRIPLE_DES_INIT_VECTOR	triple_des;
	AEP_PLAIN_DES_INIT_VECTOR	plain_des;
}	AEP_MECHANISM_DATA;

typedef struct AEP_MECHANISM {
	AEP_U32				type;
	AEP_MECHANISM_DATA	data;
}	AEP_MECHANISM;

/* End of Structs for mechansim. */
/*****************************************************************/
#endif
